;*******************************************************************************
;
;                               PIC2DHTI2C
;
;*******************************************************************************
;
;
; PIC12F1840 для нескольких DHT22 и
; для работы PIC12F1840 в качестве ведомого устройства I2C
;
; Программа использует I2C код, основанный на AN734C и модифицированный для 12F1840
;*******************************************************************************
;-------------------------------------------------------------------------------

	list		p=12F1840      ; директива list для определения процессора
	#include	<p12F1840.inc> ; определения переменных, зависимых от процессора
    errorlevel  -302           ; 
    errorlevel  -303           ;
;
; CONFIGS
;_FOSC_INTOSC    Генератор INTOSC (16 МГц:) Вывод CLKIN работает как вход/выход
;_WDTE_ON        WDT включен
;_PWRTE_OFF      PWRT выключен <--
;_MCLRE_OFF      Вывод MCLR/VPP работает как цифровой вход <--
;_CP_OFF         Защита кода памяти программ выключена <--
;_CPD_OFF        Защита кода памяти данных выключена <--
;_BOREN_OFF      Сброс при снижении напряжения питания выключен
;_CLKOUTEN_OFF   Функционал CLKOUT выключен. На выводе CLKOUT - ввод/вывод или генератор
;_IESO_ON        Режим переключения внутренний/внешний включен
;_FCMEN_ON       Безопасный режим мониторинга тактового генератора включен
    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_ON & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_ON & _FCMEN_ON
;_WRT_OFF        Защита записи выключена
;_PLLEN_ON       PLL включен
;_STVREN_OFF     Переполнение или опустошение стека не будет вызывать сброс
;_BORV_19        Напряжение сброса при просадке (Vbor), выбрана нижняя точка выключения
;_LVP_OFF        Высокое напряжение на MCLR/VPP должно использоваться для программирования
    __CONFIG _CONFIG2, _WRT_OFF & _PLLEN_ON & _STVREN_OFF & _BORV_19 & _LVP_OFF
;
#define I2C_ADDRESS 	0x32				; адрес ведомого
#define	RX_ELEMENTS		0x20             	; количество доступных элементов массива, в данном случае 32
; Это биты данных для трех DHT22
; RA1=SCL / RA2=SDA / RA3=MCLR только на вход
DQ1      EQU RA0                             ; DHT22_1 DQ на RA0
DQ2      EQU RA4                             ; DHT22_2 DQ на RA4
DQ3      EQU RA5                             ; DHT22_3 DQ на RA5
;------------------------------------ переменные --------------------------------------------------
; Примечание они помещаются в общую RAM, поэтому нам не нужно выбирать банк BANKSEL,
; но помните, что у нас только 16 байтов
    cblock 0x70                             ; установить в общей памяти для простого доступа
		INDEX_I2C							; индекс, используемый для указания положения массива
		FIRST								; used to determine if it is first data byte
        d0                                  ; счетчика задержек
        d1
        d2
        bytecnt                             ; счетчик байтов
        bitcnt                              ; счетчик битов
    endc
; Байты I2C_ARRAY
; [0] = байт запроса, устанавливается мастером. Инициализируется в 0 микроконтроллером PIC
;
; Ведомый [PIC] устанавливает [0]
;            [0x00] значение старта/сброса
;            [0x01] успешное выполнение команды
;            [0xf1] неопределенная ошибка или плохая/невыполнимая команда
; Мастер [Galileo] устанавливает [0] для запроса действий
;            [0x02] прочитать DHT #1
;            [0x03] прочитать DHT #2
;            [0x04] прочитать DHT #3
;            [0x10] сбросить PIC
; [1]-[5]   будут хранить данные DHT #1
; [6]-[10]  будут хранить данные DHT #2
; [11]-[15] будут хранить данные DHT #3
; [16]-[31] свободные и доступные
;-------------------
; 5 байтов информации DHT представляют следующее:
; влажность, целая часть
; влажность, часть после запятой
; температура, целая часть
; температура, часть после запятой
; контрольная сумма – это суммпа всех четырех байтов AND 255
; -------------------
;
; I2C_ARRAY будет заканчиваться в 0x20, поэтому следите за банками!
; irq будет хранить BSR для банка I2C/SSP, поэтому следите за остальным
    udata
I2C_ARRAY   res RX_ELEMENTS              ; массив для записи, размер памяти массива
;
;------------------------------------------------------------------------------
; Инициализация EEPROM
;
; 12F1840 имеет 256 байтов энергонезависимой EEPROM, начиная с адреса 0xF000
;
;------------------------------------------------------------------------------
DATAEE    ORG  0xF000
    DE    "DrGee"  ; Поместить 'D' 'r' 'G' 'e' 'e'  в адрес 0,1,2,3,5
;
;------------------------------------------------------------------------------
; Макросы
;-----------------------------------------------------------------------------------------------
; Макрос LOADFSR загружает FSR1 адресом I2C и значением индекса I2C для чтения или записи
; и делает код более простым для чтения.
LOADFSR macro 	ADDRESS,INDEX 				; ADDRESS = I2C_ARRAY, INDEX = INDEX_I2C
		movlw 	ADDRESS 					; загрузить адрес
		addwf	INDEX,W						; добавить значение индекса для определения места в массиве
		movwf 	FSR1L						; загрузить FSR1L с информацией указателя
		clrf	FSR1H
		endm
;-----------------------------------------------------------------------------------------------
;
;----------------------------------- вектор старта ------------------------------
 		ORG    	0x0000          			; вектор сброса
    	goto   	Main                    	; перейти к основному коду
;----------------------------------- вектор прерывания --------------------------
    	ORG    	0x0004          			; вектор прерывания
		banksel	PIR1
		btfss 	PIR1,SSP1IF 				; это прерывание SSP?
		goto 	BUS_COLL 					; если нет, произошла коллизия шины
    	banksel	SSPSTAT
		btfsc	SSPSTAT,2					; это читает мастер:
		goto	READ						; если да, то идем сюда
		goto	WRITE						; если нет, то идем сюда
;
;-------------------------------------------------------------------------------
;	                              Основной код
;-------------------------------------------------------------------------------
		ORG		0x0040        				; начало флеш памяти
Main
		call	INITIALIZE					; настройка микроконтроллера
        movlw   0x00
        movwf   I2C_ARRAY                   ; очистить команду array[0]
LOOP
		clrwdt                              ; очистить WDT
;
; команды / статус в I2C_ARRAY[0]
; сначала проверить на допустимые значения статуса и просто зациклить до запроса команды
        movf	I2C_ARRAY,0     ; array[0] в W
        iorlw   0x0             ; array[0]=0x00 не команда / состояние сброса
        btfsc   STATUS,Z
        goto    LOOP
        movlw   0x01            ; array[0]=0x01 не команда [последняя команда выполнена успешно)
        subwf   I2C_ARRAY,W
        btfsc   STATUS,Z
        goto    LOOP
        movlw   0xf1            ; array[0]=0xf1 неудачная команда на последней операции
        subwf   I2C_ARRAY,W
        btfsc   STATUS,Z
        goto    LOOP
; проверить допустимость команды
        movlw   0x02            ; array[0]=0x02 - прочитать DHT #1
        subwf   I2C_ARRAY,W
        btfsc	STATUS, Z		; проверить, если 0..
        goto    ReadDHT1
        movlw   0x03            ; array[0]=0x3 - прочитать DHT #2
        subwf   I2C_ARRAY,W
        btfsc	STATUS, Z		; проверить, если 0..
        goto    ReadDHT2
        movlw   0x04            ; array[0]=0x04 - прочитать DHT #3
        subwf   I2C_ARRAY,W
        btfsc	STATUS, Z		; проверить, если 0..
        goto    ReadDHT3
        movlw   0x10            ; array[0]=0x10 - сбросить PIC
        subwf   I2C_ARRAY,W
        btfsc	STATUS, Z		; проверить, если 0..
        goto    ResetPIC
; пройти до неудачной команды, если не было найдено ничего допустимого
;----- BADCMD
BADCMD
; байт команды был не распознан - поместить 0xf1 в [0]
        movlw   0xf1                        ; код недопустимой команды
        movwf   I2C_ARRAY                   ; в [0]
        goto    LOOP
CMDDONE
; успешное выполнение команды
; успех указывается установкой [0]=1
        movlw   0x01
        movwf   I2C_ARRAY
        goto    LOOP
;---------------------------------------------------------------------------
; Прочитать первый DHT-22 и сохранить 5 байтов данных в DHT
ReadDHT1
; настроить для передачи в I2C_ARRAY с +1 до +5
    movlw   0x05            ; байты из DHT
    movwf   bytecnt
    movlw   0x08            ; биты
    movwf   bitcnt
    movlw   (I2C_ARRAY+1)   ; точка начала буфера в ARRAY
    movwf 	FSR0L			; загрузить FSR0L с информацией об указателе
	clrf	FSR0H
; инициализировать DHT
Init_HU_1
; настроить вывод данных на выход и установить на нем низкий уровень
   banksel LATA
   bcf     LATA,DQ1
   banksel TRISA
   bcf     TRISA,DQ1
; задержка 18 мкс
	movlw 0x3F
	movwf d0
	movlw 0x39
	movwf d1
D18ms_1
	decfsz d0, f
	goto  $+2
	decfsz d1, f
	goto D18ms_1
	goto  $+1
; установить высокий уровень на линии данных на 30 мкс
   banksel LATA
   bsf     LATA,DQ1
; задержка на 30 мкс (20-40 мкс по техническому описанию)
	movlw 0x27
	movwf d0
D30us_1
	decfsz d0, f
	goto D30us_1
	goto  $+1
; теперь DHT будет передавать данные...
; настроить вывод данных на вход
    banksel LATA
    bcf     LATA,DQ1
    banksel TRISA
    bsf     TRISA,DQ1
; DHT должен установить низкий уровень на 80 мкс, но проверьте и ждите, пока он низкий
wlow_1
    banksel PORTA
    btfsc   PORTA,DQ1
    goto    wlow_1
; затем DHT должен установить на линии данных высокий уровень на 80 мкс
whigh_1
    btfss   PORTA,DQ1
    goto    whigh_1
; теперь ждать снова низкого уровня, отметив начало стартового бита
wlow2_1
    btfsc   PORTA,DQ1
    goto    wlow2_1
; здесь стартовый бит - 50 мкс низкого уровня на линии данных, за которым следует
; высокий уровень 26-28 мкс (если "0") или 70 мкс (если "1")
start_bit_1
; ждать окончания стартового бита
whigh2_1
    btfss   PORTA,DQ1
    goto    whigh2_1
; а затем задержка на 40 мкс
	movlw 0x35
	movwf d0
D40us_1
	decfsz d0, f
	goto D40us_1
; теперь прочитать состояние на линии данных, чтобы определить, что на ней 0 или 1
; обратите внимание, что мы всё еще в банке PORTA
read_bit_1
    btfsc   PORTA,DQ1
    goto    one_bit_1
; бит был "0", и он закончился, и сейчас идет стартовый бит
; с оставшимися ~24 мкс - обработать бит и перейти к ожиданию следующего высокого уровня
    bcf     STATUS, C
    rlf     INDF0,1
; декрементировать счетчик битов
dec_bit_1
    decfsz    bitcnt,f
    goto      no_bump_1
; сбросить bit_count и декрементировать bytcount
    movlw   0x08
    movwf   bitcnt
; указать на следующую ячейку данных 
    incf    FSR0
    decfsz  bytecnt,f
    goto    no_bump_1
; здесь у нас есть все 5 байтов данных, поэтому передача закончена
    goto fin_trans_1
no_bump_1
    goto    whigh2_1
one_bit_1
; бит был "1", и мы всё еще в стартовом бите - ждем его завершения
; затем обработать бит и проверить линию на фронт и спад в wait_high2
wait_for_end_1
    btfsc   PORTA,DQ1
    goto    wait_for_end_1
; обработать бит
    bsf     STATUS, C
    rlf     INDF0,1
    goto    dec_bit_1
;
fin_trans_1
; мы прочитали показания датчика и все 5 байтов загружены в ARRAY[1]-[5]
    goto    CMDDONE
;
; Прочитать второй DHT-22 и сохранить 5 байтов данных в DHT
ReadDHT2
; настроить для передачи в I2C_ARRAY с +6 до +10
    movlw   0x05            ; байты из DHT
    movwf   bytecnt
    movlw   0x08            ; биты
    movwf   bitcnt
    movlw   (I2C_ARRAY+6)   ; точка начала буфера в ARRAY
    movwf 	FSR0L			; загрузить FSR0L с информацией об указателе
	clrf	FSR0H
; инициализировать DHT
Init_HU_2
; настроить вывод данных на выход и установить на нем низкий уровень
   banksel LATA
   bcf     LATA,DQ2
   banksel TRISA
   bcf     TRISA,DQ2
; задержка 18 мкс
	movlw 0x3F
	movwf d0
	movlw 0x39
	movwf d1
D18ms_2
	decfsz d0, f
	goto  $+2
	decfsz d1, f
	goto D18ms_2
	goto  $+1
; установить высокий уровень на линии данных на 30 мкс
   banksel LATA
   bsf     LATA,DQ2
; задержка на 30 мкс (20-40 мкс по техническому описанию)
	movlw 0x27
	movwf d0
D30us_2
	decfsz d0, f
	goto D30us_2
	goto  $+1
; теперь DHT будет передавать данные...
; настроить вывод данных на вход
    banksel LATA
    bcf     LATA,DQ2
    banksel TRISA
    bsf     TRISA,DQ2
; DHT должен установить низкий уровень на 80 мкс, но проверьте и ждите, пока он низкий
wlow_2
    banksel PORTA
    btfsc   PORTA,DQ2
    goto    wlow_2
; затем DHT должен установить на линии данных высокий уровень на 80 мкс
whigh_2
    btfss   PORTA,DQ2
    goto    whigh_2
; теперь ждать снова низкого уровня, отметив начало стартового бита
wlow2_2
    btfsc   PORTA,DQ2
    goto    wlow2_2
; здесь стартовый бит - 50 мкс низкого уровня на линии данных, за которым следует
; высокий уровень 26-28 мкс (если "0") или 70 мкс (если "1")
start_bit_2
; ждать окончания стартового бита
whigh2_2
    btfss   PORTA,DQ2
    goto    whigh2_2
; а затем задержка на 40 мкс
	movlw 0x35
	movwf d0
D40us_2
	decfsz d0, f
	goto D40us_2
; теперь прочитать состояние на линии данных, чтобы определить, что на ней 0 или 1
; обратите внимание, что мы всё еще в банке PORTA
read_bit_2
    btfsc   PORTA,DQ2
    goto    one_bit_2
; бит был "0", и он закончился, и сейчас идет стартовый бит
; с оставшимися ~24 мкс - обработать бит и перейти к ожиданию следующего высокого уровня
    bcf     STATUS, C
    rlf     INDF0,1
; декрементировать счетчик битов
dec_bit_2
    decfsz    bitcnt,f
    goto      no_bump_2
; сбросить bit_count и декрементировать bytcount
    movlw   0x08
    movwf   bitcnt
; указать на следующую ячейку данных 
    incf    FSR0
    decfsz  bytecnt,f
    goto    no_bump_2
; здесь у нас есть все 5 байтов данных, поэтому передача закончена
    goto fin_trans_2
no_bump_2
    goto    whigh2_2
one_bit_2
; бит был "1", и мы всё еще в стартовом бите - ждем его завершения
; затем обработать бит и проверить линию на фронт и спад в wait_high2
wait_for_end_2
    btfsc   PORTA,DQ2
    goto    wait_for_end_2
; обработать бит
    bsf     STATUS, C
    rlf     INDF0,1
    goto    dec_bit_2
;
fin_trans_2
; мы прочитали показания датчика и все 5 байтов загружены в ARRAY[6]-[10]
    goto    CMDDONE
;
ReadDHT3
; Прочитать третий DHT-22 и сохранить 5 байтов данных в DHT
;
; настроить для передачи в I2C_ARRAY с +11 до +15
    movlw   0x05            ; байты из DHT
    movwf   bytecnt
    movlw   0x08            ; биты
    movwf   bitcnt
    movlw   (I2C_ARRAY+0x0b); точка начала буфера в ARRAY
    movwf 	FSR0L			; загрузить FSR0L с информацией об указателе
	clrf	FSR0H
; инициализировать DHT
Init_HU_3
; настроить вывод данных на выход и установить на нем низкий уровень
    banksel LATA
    bcf     LATA,DQ3
    banksel TRISA
    bcf     TRISA,DQ3
; задержка 18 мкс
	movlw 0x3F
	movwf d0
	movlw 0x39
	movwf d1
D18ms_3
	decfsz d0, f
	goto  $+2
	decfsz d1, f
	goto D18ms_3
	goto  $+1
; установить высокий уровень на линии данных на 30 мкс
   banksel LATA
   bsf     LATA,DQ3
; задержка на 30 мкс (20-40 мкс по техническому описанию)
	movlw 0x27
	movwf d0
D30us_3
	decfsz d0, f
	goto D30us_3
	goto  $+1
; теперь DHT будет передавать данные...
; настроить вывод данных на вход
    banksel LATA
    bcf     LATA,DQ3
    banksel TRISA
    bsf     TRISA,DQ3
; DHT должен установить низкий уровень на 80 мкс, но проверьте и ждите, пока он низкий
wlow_3
    banksel PORTA
    btfsc   PORTA,DQ3
    goto    wlow_3
; затем DHT должен установить на линии данных высокий уровень на 80 мкс
whigh_3
    btfss   PORTA,DQ3
    goto    whigh_3
; теперь ждать снова низкого уровня, отметив начало стартового бита
wlow2_3
    btfsc   PORTA,DQ3
    goto    wlow2_3
; здесь стартовый бит - 50 мкс низкого уровня на линии данных, за которым следует
; высокий уровень 26-28 мкс (если "0") или 70 мкс (если "1")
start_bit_3
; ждать окончания стартового бита
whigh2_3
    btfss   PORTA,DQ3
    goto    whigh2_3
; а затем задержка на 40 мкс
	movlw 0x35
	movwf d0
D40us_3
	decfsz d0, f
	goto D40us_3
; теперь прочитать состояние на линии данных, чтобы определить, что на ней 0 или 1
; обратите внимание, что мы всё еще в банке PORTA
read_bit_3
    btfsc   PORTA,DQ3
    goto    one_bit_3
; бит был "0", и он закончился, и сейчас идет стартовый бит
; с оставшимися ~24 мкс - обработать бит и перейти к ожиданию следующего высокого уровня
    bcf     STATUS, C
    rlf     INDF0,1
; декрементировать счетчик битов
dec_bit_3
    decfsz  bitcnt,f
    goto    no_bump_3
; сбросить bit_count и декрементировать bytcount
    movlw   0x08
    movwf   bitcnt
; указать на следующую ячейку данных 
    incf    FSR0
    decfsz  bytecnt,f
    goto    no_bump_3
; здесь у нас есть все 5 байтов данных, поэтому передача закончена
    goto    fin_trans_3
no_bump_3
    goto    whigh2_3
one_bit_3
; бит был "1", и мы всё еще в стартовом бите - ждем его завершения
; затем обработать бит и проверить линию на фронт и спад в wait_high2
wait_for_end_3
    btfsc   PORTA,DQ3
    goto    wait_for_end_3
; обработать бит
    bsf     STATUS, C
    rlf     INDF0,1
    goto    dec_bit_3
;
fin_trans_3
; мы прочитали показания датчика и все 5 байтов загружены в ARRAY[11]-[15]
    goto    CMDDONE
;
;---------------------------------------------------------------------------
;
;----- Сброс PIC
; выполнить программный эквивалент аппаратного сброса 
; это просто аварийная помощь, которая не должна использоваться
ResetPIC
    RESET
;------------------------------------- конец main ------------------------------------------------
;
;-----------------------------------------------------------------------------------------------
;	Инициализация: настройка значений регистров
;-----------------------------------------------------------------------------------------------
INITIALIZE
;настройка микроконтроллера
; настроить генератор на 16 МГц
;    movlw   b'01111000'
;              1        4x PLL включен
;               1111    16 МГц внутренний генератор
;                   x   
;                    1  блок внутреннего генератора
;                     0  из конфига
;                     x  когда 1=1
;Заметьте, что 4x PLL включен, как требует CONFIG
		banksel	OSCCON
		movlw	b'01111010'					; Внутренний генератор @ 16 МГц, выбран блок внутреннего генератора
		movwf	OSCCON                      ; 4xPLL включен (установлен в слове конфига)
		movlw	b'11010111'                 ; WPU выключен, INT по нарастающему фронту, FOSC/4,
		movwf	OPTION_REG					; Предделитель назначен на TMR0, коэффициент 1:256
        movlw   b'00011011'                 ; Сторожевой таймер на 8 сек, swdten установлен
		movwf	WDTCON
; установить все входы/выходы на вход
        banksel PORTA
        clrf    PORTA       ; инициализировать porta
        banksel LATA
        clrf    LATA        ; защелка данных
        banksel ANSELA
        clrf    ANSELA      ; ВНИМАНИЕ настройка на цифровой ввод/вывод ***********
        banksel TRISA
; установить всё на вход 
        movlw   b'111111'
        movwf   TRISA
		clrf	INDEX_I2C                   ; очистить указатель индекса 
		clrf	FIRST                       ; очистить FIRST
		bsf		FIRST,0                     ; установить бит 0 в FIRST
; настройка I2C
		banksel	SSPSTAT
		bsf		SSPSTAT,SMP					; Контроль скорости выключен для стандартного режима скорости 
		movlw	b'00110110'					; Включить последовательный порт, I2C режим ведомый, 7-битный адрес
		movwf	SSPCON1
		bsf		SSPCON2,SEN					; включить растяжение тактов
		bsf		SSPCON3,BOEN				; SSPBUF обновлен и NACK генерируется
											; для принятого байта адреса/данных,
											; игнорируя состояние бита SSPOV, только если бит BF = 0.
		bsf		SSPCON3,SDAHT				; Минимальное время ожидания 300 нс
		movlw	I2C_ADDRESS					; Загрузить адрес ведомого
		movwf	SSPADD
		banksel	PIR1
		bcf		PIR1,SSP1IF					; очистить флаг прерывания SSP
		banksel	PIE1
		bsf		PIE1,SSP1IE					; включить SSP прерывания
		banksel	PIR2
		bcf		PIR2,BCL1IF					; очистить флаг прерывания SSP
		banksel	PIE2
		bsf		PIE2,BCL1IE					; включить SSP прерывания
;set up the interrupt
        banksel INTCON
		bsf		INTCON,PEIE					; включить периферийные прерывания
		bsf		INTCON,GIE					; включить глобальные прерывания
		return
;------------------------------------- Конец инициализации ------------------------------------------
;
; Код ниже взят из Microchip AN734C
;
; Software License Agreement
;
; The software supplied herewith by Microchip Technology Incorporated
; (the 'Company') is intended and supplied to you, the Company's customer,
; for use solely and exclusively with products manufactured by the Company.
; The software is owned by the Company and/or its supplier, and is protected
; under applicable copyright laws. All rights are reserved. Any use in
; violation of the foregoing restrictions may subject the user to criminal
; sanctions under applicable laws, as well as to civil liability for the breach
; of the terms and conditions of this license.
;
; THIS SOFTWARE IS PROVIDED IN AN 'AS IS' CONDITION. NO WARRANTIES,
; WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
; APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE
; LIABLE FOR SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES, FOR ANY REASON
; WHATSOEVER.

;-----------------------------------------------------------------------------------------------
;                                Interrupt Service Routine (ISR)
;-----------------------------------------------------------------------------------------------
READ
		banksel	SSPSTAT
		btfss	SSPSTAT,5					; was last byte an address or data?
		goto	R_ADDRESS					; if clear, it was an address
		goto	R_DATA						; if set, it was data
;
R_ADDRESS
		LOADFSR	I2C_ARRAY,INDEX_I2C			; call LOADFSR macro
		movf	INDF1,W						; move value into W to load to SSP buffer
		banksel	SSPBUF
		movwf	SSPBUF						; load SSP buffer
		incf	INDEX_I2C,F					; increment INDEX_I2C 'pointer'
        banksel	SSPCON1
		btfsc	SSPCON1,WCOL				; did a write collision occur?
		goto  	WRITE_COLL					; if so, go clear bit
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
R_DATA
		LOADFSR	I2C_ARRAY,INDEX_I2C			; call LOADFSR macro
		movf	INDF1,W						; move value into W to load to SSP buffer
		banksel	SSPBUF
		movwf	SSPBUF						; load SSP buffer
		incf	INDEX_I2C,F					; increment INDEX_I2C 'pointer'
        banksel	SSPCON1
		btfsc	SSPCON1,WCOL				; did a write collision occur?
		goto  	WRITE_COLL					; if so, go clear bit
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
WRITE
		banksel	SSPSTAT
		btfss	SSPSTAT,5					; was last byte an address or data?
		goto	W_ADDRESS					; if clear, it was an address
		goto	W_DATA						; if set, it was data

W_ADDRESS
        bsf		FIRST,0						; set bit 0 of FIRST
		banksel	SSPBUF
		movfw	SSPBUF						; move the contents of the buffer into W
                                            ; dummy read to clear the BF bit
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt

W_DATA
		btfss	FIRST,0						; is bit 0 set?
		goto	W_DATA_SUB					; if clear, go write data
		banksel	SSPBUF						; if set, first data byte is location to write to
		movfw	SSPBUF
		movwf	INDEX_I2C					; load INDEX_I2C with data address to write to
		clrf	FIRST						; clear the FIRST register
        banksel	SSPCON1
		btfsc	SSPCON1,WCOL				; did a write collision occur?
		goto  	WRITE_COLL					; if so, go clear bit
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt

W_DATA_SUB
		clrf	W                           ; clear W
		movlw	RX_ELEMENTS					; load array elements value
		banksel STATUS
		btfsc	STATUS,Z					; is Z clear?
		subwf	INDEX_I2C,W					; if Z = 1, subtract index from number of elements
		banksel	STATUS
		btfsc	STATUS,0					; did a carry occur after subtraction?
		goto	NO_MEM_OVERWRITE			; if so, Master is trying to write to many bytes
		LOADFSR	I2C_ARRAY,INDEX_I2C			; call LOADFSR macro
		banksel	SSPBUF
		movfw	SSPBUF						; move the contents of the buffer into W
		movwf 	INDF1						; load INDF1 with data to write
		incf	INDEX_I2C,F					; increment 'pointer' to next memory location
        banksel	SSPCON1
		btfsc	SSPCON1,WCOL				; did a write collision occur?
		goto  	WRITE_COLL					; if so, go clear bit
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
NO_MEM_OVERWRITE
		banksel	SSPBUF
		movfw	SSPBUF						; move SSP buffer to W
                    						; clear buffer so no overwrite occurs
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
WRITE_COLL
		banksel	SSPCON1
		bcf		SSPCON1,WCOL				; clear WCOL bit
		banksel	SSPBUF
		movfw	SSPBUF						; move SSP buffer to W
                    						; clear SSP buffer
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		banksel	PIR1
		bcf 	PIR1,SSP1IF					; clear the SSP interrupt flag
		goto    EXIT 						; Go to EXIT to return from interrupt
;
BUS_COLL
		banksel	SSPBUF
		clrf	SSPBUF						; clear the SSP buffer
		banksel	PIR2
		bcf		PIR2,BCL1IF					; clear the SSP interrupt flag
		banksel	SSPCON1
		bsf		SSPCON1,CKP					; release clock stretch
		goto    EXIT						; Go to EXIT to return from interrupt
;
EXIT
        retfie                              ; Return from interrupt
;
;##############################################################################
		end									; END OF PROGRAM





